@@ -34,7 +34,7 @@
 #include <glib/gi18n-lib.h>
 
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
+
 #include <libgnomeui/libgnomeui.h>
 
 #include <libgnomevfs/gnome-vfs.h>
@@ -43,21 +43,17 @@
 
 #include <nautilus-burn-drive.h>
 
+#include <gst/gst.h>
+
 #include "utils.h"
 #define BRASERO_ERROR brasero_error_quark()
 
-static GHashTable *icons = NULL;
 static gid_t *groups = NULL;
 G_LOCK_DEFINE (groups_mutex);
 
 static void
 brasero_utils_free (void)
 {
-	if (icons) {
-		g_hash_table_destroy (icons);
-		icons = NULL;
-	}
-
 	if (groups) {
 		G_LOCK (groups_mutex);
 		g_free (groups);
@@ -77,33 +73,15 @@ brasero_error_quark (void)
 	return quark;
 }
 
-
-/* utils */
 void
 brasero_utils_init (void)
 {
-	GdkPixbuf *pixbuf;
-	GtkIconFactory *factory;
-
 	/* This function does two things:
 	 * - register brasero icons
 	 * - load all the gid of the user
 	 */
-	factory = gtk_icon_factory_new ();
-	gtk_icon_factory_add_default (factory);
-
-	pixbuf = gdk_pixbuf_new_from_file (BRASERO_DATADIR G_DIR_SEPARATOR_S "icon-final-48x48.png", NULL);
-	if (pixbuf) {
-		GtkIconSet *iconset;
-
-		iconset = gtk_icon_set_new_from_pixbuf (pixbuf);
-		g_object_unref (pixbuf);
-
-		gtk_icon_factory_add (factory, BRASERO_STOCK_ICON, iconset);
-		gtk_icon_set_unref (iconset);
-	}
-	else
-		g_warning ("icon %s can't be loaded\n", BRASERO_DATADIR G_DIR_SEPARATOR_S "icon-final-48x48.png");
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
+					   BRASERO_DATADIR "/icons");
 
 	/* load gids of the user */
 	if (groups == NULL) {
@@ -130,7 +108,7 @@ brasero_utils_is_gid_in_groups (gid_t gid)
 		return FALSE;
 	}
 
-	for (group = groups; group; group ++) {
+	for (group = groups; group && *group; group ++) {
 		if (*group == gid) {
 			G_UNLOCK (groups_mutex);
 			return TRUE;
@@ -141,43 +119,73 @@ brasero_utils_is_gid_in_groups (gid_t gid)
 	return FALSE;
 }
 
-char *
-brasero_utils_get_time_string (double time, gboolean with_unit)
+gchar *
+brasero_utils_get_time_string (gint64 time,
+			       gboolean with_unit,
+			       gboolean round)
 {
 	int second, minute, hour;
 
 	time = time / 1000000000;
 	hour = time / 3600;
-	time = ((int) time) % 3600;
+	time = time % 3600;
 	minute = time / 60;
-	second = (int) time % 60;
+
+	if (round) {
+		if ((time % 60) > 30)
+			minute ++;
+
+		second = 0;
+	}
+	else
+		second = time % 60;
 
 	if (hour) {
-		if (with_unit)
-			return g_strdup_printf ("%i h %i min %02i ", hour, minute, second);
-		else
-			return g_strdup_printf ("%i:%i:%02i", hour, minute, second);
+		if (with_unit) {
+			if (hour && minute && second)
+				return g_strdup_printf ("%i h %02i min %02i",
+							 hour,
+							 minute,
+							 second);
+			else if (hour && minute)
+				return g_strdup_printf ("%i h %02i",
+							 hour,
+							 minute);
+			else
+				return g_strdup_printf ("%i h",hour);
+		}
+		else if (hour && minute && second)
+			return g_strdup_printf ("%i:%02i:%02i",
+						 hour,
+						 minute,
+						 second);
+		else if (hour && minute)
+			return g_strdup_printf ("%i:%02i", hour, minute);
 	}
 
-	if (with_unit)
-		return g_strdup_printf ("%i:%02i min", minute, second);
+	if (with_unit) {
+		if (!second)
+			return g_strdup_printf (_("%i min"), minute);
+		else
+			return g_strdup_printf (_("%i:%02i min"), minute, second);
+	}
 	else
 		return g_strdup_printf ("%i:%02i", minute, second);
 }
 
-char *
-brasero_utils_get_time_string_from_size (double size,
+gchar *
+brasero_utils_get_time_string_from_size (gint64 size,
 					 gboolean with_unit,
 					 gboolean round)
 {
 	int second = 0;
 	int minute = 0;
-	double time = 0.0;
+	gint64 time = 0.0;
 
-	time = (double) NAUTILUS_BURN_DRIVE_SIZE_TO_TIME (size);
+	time = NAUTILUS_BURN_DRIVE_SIZE_TO_TIME (size);
 	minute = time / 60;
 	if (!round)
-		second = (int) time % 60;
+		second = time % 60;
 
 	if (minute > 120) {
 		int hour;
@@ -185,18 +193,18 @@ brasero_utils_get_time_string_from_size (double size,
 		hour = minute / 60;
 		minute = minute % 60;
 
-		if (with_unit == TRUE) {
-			if (hour == 1)
-				return g_strdup_printf (_("1:%02i hour"),
-							minute);
-			else
-				return g_strdup_printf (_("%i:%02i hours"),
-							hour, minute);
-		} else
+		if (with_unit == TRUE)
+			return g_strdup_printf (ngettext ("%d:%02i hour", "%i:%02i hours", hour), hour, minute);
+		else
 			return g_strdup_printf ("%i:%02i:%02i", hour,
 						minute, second);
-	} else if (with_unit == TRUE)
-		return g_strdup_printf ("%i:%02i min", minute, second);
+	}
+	else if (with_unit == TRUE) {
+		if (!second)
+			return g_strdup_printf (_("%i min"), minute);
+		else
+			return g_strdup_printf (_("%i:%02i min"), minute, second);
+	}
 	else
 		return g_strdup_printf ("%i:%02i", minute, second);
 }
@@ -208,18 +216,17 @@ enum {
 	BRASERO_UTILS_GO
 };
 
-
-char *
-brasero_utils_get_size_string (double dsize,
+gchar *
+brasero_utils_get_size_string (gint64 dsize,
 			       gboolean with_unit,
 			       gboolean round)
 {
 	int unit;
 	int size;
 	int remain = 0;
-	const char *units[] = { "", N_("Kb"), N_("Mb"), N_("Gb") };
+	const char *units[] = { "", N_("KiB"), N_("MiB"), N_("GiB") };
 
-	if (dsize < 1024.0) {
+	if (dsize < 1024) {
 		unit = BRASERO_UTILS_NO_UNIT;
 		size = (int) dsize;
 		goto end;
@@ -270,74 +277,26 @@ brasero_utils_get_size_string (double dsize,
 					size);
 }
 
-GdkPixbuf *
-brasero_utils_get_icon (const char *name, int size)
+gchar *
+brasero_utils_get_sectors_string (gint64 sectors,
+				  gboolean time_format,
+				  gboolean with_unit,
+				  gboolean round)
 {
-	char *real;
-	GError *error = NULL;
-	GdkPixbuf *icon_pix = NULL;
+	gint64 size;
 
-	if (!name)
-		return NULL;
-
-	/* fetch the icon in default factory */
-	if (icons == NULL)
-		icons = g_hash_table_new_full (g_str_hash,
-					       g_str_equal,
-					       (GDestroyNotify) g_free,
-					       (GDestroyNotify) g_object_unref);
-
-	real = g_strdup_printf ("%s-%i", name, size);
-	icon_pix = g_hash_table_lookup (icons, real);
-	if (!icon_pix) {
-		icon_pix = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-						     name,
-						     size,
-						     0,
-						     &error);
-
-		if (!icon_pix) {
-			if (error) {
-				g_warning ("Couldn't load icon: %s\n", error->message);
-				g_error_free (error);
-			}
-			g_free (real);
-			return NULL;
-		}
-		else if (icons)
-			g_hash_table_insert (icons,
-					     real,
-					     icon_pix);
+	if (time_format) {
+		size = sectors * GST_SECOND / 75;
+		return brasero_utils_get_time_string (size, with_unit, round);
 	}
-	else
-		g_free (real);
-
-	g_object_ref (icon_pix);
-	return icon_pix;
-}
-
-GdkPixbuf *
-brasero_utils_get_icon_for_mime (const char *mime, int size)
-{
-	char *icon_string;
-	GdkPixbuf *pixbuf;
-
-	icon_string = gnome_icon_lookup (gtk_icon_theme_get_default (), NULL,
-					 NULL, NULL, NULL, mime,
-					 GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
-
-	if (!icon_string) {
-		g_warning ("Couldn't load icon:\n no icon registered for this type.\n");
-		return NULL;
+	else {
+		size = sectors * 2048;
+		return brasero_utils_get_size_string (size, with_unit, round);
 	}
-
-	pixbuf = brasero_utils_get_icon (icon_string, size);
-	g_free (icon_string);
-	return pixbuf;
 }
 
 GtkWidget *
-brasero_utils_pack_properties_list (const char *title, GSList *list)
+brasero_utils_pack_properties_list (const gchar *title, GSList *list)
 {
 	GtkWidget *hbox, *vbox_main, *vbox_prop;
 	GtkWidget *label;
@@ -345,18 +304,6 @@ brasero_utils_pack_properties_list (const char *title, GSList *list)
 
 	vbox_main = gtk_vbox_new (FALSE, 6);
 
-	if (title) {
-		gtk_container_set_border_width (GTK_CONTAINER (vbox_main), 6);
-		label = gtk_label_new (title);
-		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-		gtk_box_pack_start (GTK_BOX (vbox_main),
-				    label,
-				    FALSE,
-				    FALSE,
-				    0);
-	}
-
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_end (GTK_BOX (vbox_main), hbox, TRUE, TRUE, 0);
 
@@ -366,18 +313,34 @@ brasero_utils_pack_properties_list (const char *title, GSList *list)
 	vbox_prop = gtk_vbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (hbox), vbox_prop, TRUE, TRUE, 0);
 
-	for (iter = list; iter; iter = iter->next)
+	for (iter = list; iter; iter = iter->next) {
 		gtk_box_pack_start (GTK_BOX (vbox_prop),
 				    iter->data,
 				    TRUE,
 				    TRUE,
 				    0);
+	}
+
+	if (title) {
+		GtkWidget *frame;
+
+		frame = gtk_frame_new (title);
+		gtk_widget_show (frame);
+		gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
+
+		label = gtk_frame_get_label_widget (GTK_FRAME (frame));
+		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+
+		gtk_container_set_border_width (GTK_CONTAINER (vbox_main), 8);
+		gtk_container_add (GTK_CONTAINER (frame), vbox_main);
+		return frame;
+	}
 
 	return vbox_main;
 }
 
 GtkWidget *
-brasero_utils_pack_properties (const char *title, ...)
+brasero_utils_pack_properties (const gchar *title, ...)
 {
 	va_list vlist;
 	GtkWidget *child;
@@ -396,37 +359,31 @@ brasero_utils_pack_properties (const char *title, ...)
 }
 
 GtkWidget *
-brasero_utils_make_button (char *text, char *stock)
+brasero_utils_make_button (const gchar *text,
+			   const gchar *stock,
+			   const gchar *theme, 
+			   GtkIconSize size)
 {
+	GtkWidget *image = NULL;
 	GtkWidget *button;
-	GtkWidget *box;
 
-	box = gtk_hbox_new (FALSE, 6);
-	gtk_box_pack_start (GTK_BOX (box),
-			    gtk_image_new_from_stock (stock, GTK_ICON_SIZE_BUTTON),
-			    FALSE,
-			    FALSE,
-			    0);
+	if (theme)
+		image = gtk_image_new_from_icon_name (theme, size);
 
-	if (text)
-		gtk_box_pack_start (GTK_BOX (box),
-				    gtk_label_new (text),
-				    FALSE,
-				    FALSE,
-				    0);
+	if (!image && stock)
+		image = gtk_image_new_from_stock (stock, size);
 
 	button = gtk_button_new ();
-	gtk_container_add (GTK_CONTAINER (button), box);
-	gtk_button_set_alignment (GTK_BUTTON (button), 0.5, 0.5);
-
+	gtk_button_set_image (GTK_BUTTON (button), image);
+	gtk_button_set_label (GTK_BUTTON (button), text);
 	return button;
 }
 
 static gboolean
-brasero_utils_empty_dir (const char *uri, GnomeVFSFileInfo * info)
+brasero_utils_empty_dir (const gchar *uri, GnomeVFSFileInfo * info)
 {
 	GnomeVFSDirectoryHandle *handle;
-	char *file_uri, *name;
+	gchar *file_uri, *name;
 
 	/* NOTE : we don't follow uris as certain files are linked by content-data */
 	if (gnome_vfs_directory_open
@@ -441,7 +398,7 @@ brasero_utils_empty_dir (const char *uri, GnomeVFSFileInfo * info)
 		    || *info->name == '.')
 			continue;
 
-		name = gnome_vfs_escape_path_string (info->name);
+		name = gnome_vfs_escape_string (info->name);
 		file_uri = g_strconcat (uri, "/", name, NULL);
 		g_free (name);
 
@@ -463,7 +420,7 @@ brasero_utils_empty_dir (const char *uri, GnomeVFSFileInfo * info)
 }
 
 gboolean
-brasero_utils_remove (const char *uri)
+brasero_utils_remove (const gchar *uri)
 {
 	GnomeVFSFileInfo *info;
 	gboolean result = TRUE;
@@ -488,13 +445,13 @@ brasero_utils_remove (const char *uri)
 	return result;
 }
 
-char *
-brasero_utils_escape_string (const char *text)
+gchar *
+brasero_utils_escape_string (const gchar *text)
 {
-	char *ptr, *result;
-	int len = 1;
+	gchar *ptr, *result;
+	gint len = 1;
 
-	ptr = (char *) text;
+	ptr = (gchar *) text;
 	while (*ptr != '\0') {
 		if (*ptr == '\\' || *ptr == '=')
 			len++;
@@ -503,7 +460,7 @@ brasero_utils_escape_string (const char *text)
 		ptr++;
 	}
 
-	result = g_new (char, len);
+	result = g_new (gchar, len);
 
 	ptr = result;
 	while (*text != '\0') {
@@ -522,108 +479,6 @@ brasero_utils_escape_string (const char *text)
 	return result;
 }
 
-char *
-brasero_utils_check_for_parent_symlink (const char *uri)
-{
-	GnomeVFSFileInfo *info;
-	GnomeVFSResult result;
-	char *escaped_uri;
-	char *parent;
-	char *retval;
-	char *tmp;
-
-	info = gnome_vfs_file_info_new ();
-	parent = g_path_get_dirname (uri);
-	retval = g_strdup (uri);
-
-	while (strcmp (parent, "/")) {
-		escaped_uri = gnome_vfs_escape_host_and_path_string (parent);
-		result = gnome_vfs_get_file_info (escaped_uri,
-						  info,
-					          GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-		g_free (escaped_uri);
-
-		if (result != GNOME_VFS_OK) {
-			/* we shouldn't reached this point normally but who knows */
-			gnome_vfs_file_info_clear (info);
-			gnome_vfs_file_info_unref (info);
-			g_free (parent);
-			return retval;
-		}
-
-		/* NOTE: no need to check for broken symlinks since
-		 * we wouldn't have reached this point otherwise */
-		if (GNOME_VFS_FILE_INFO_SYMLINK (info)) {
-			char *root;
-			char *newuri;
-
-			root = parent;
-			parent = g_strconcat ("file://",
-					      info->symlink_name,
-					      NULL);
-
-			if (g_str_has_suffix (parent, "/"))
-				newuri = g_strconcat (parent,
-						      retval + strlen (root) + 1,
-						      NULL);
-			else
-				newuri = g_strconcat (parent,
-						      retval + strlen (root),
-						      NULL);
-			g_free (root);
-			g_free (retval);
-			retval = newuri;
-		}
-
-		tmp = parent;
-		parent = g_path_get_dirname (parent);
-		g_free (tmp);
-
-		gnome_vfs_file_info_clear (info);
-	}
-
-	gnome_vfs_file_info_unref (info);
-	g_free (parent);
-	return retval;
-}
-
-gboolean
-brasero_utils_get_symlink_target (const char *uri,
-				  GnomeVFSFileInfo *info,
-				  GnomeVFSFileInfoOptions flags)
-{
-	int size;
-	char *target;
-	char *escaped_uri;
-	GnomeVFSResult result;
-
-	escaped_uri = gnome_vfs_escape_host_and_path_string (uri);
-	result = gnome_vfs_get_file_info (escaped_uri,
-					  info,
-					  flags|
-					  GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-	g_free (escaped_uri);
-	if (result)
-		return FALSE;
-
-	target = g_strconcat ("file://", info->symlink_name, NULL);
-	if (g_str_has_suffix (target, "/"))
-		target [strlen (target) - 1] = '\0';
-
-	g_free (info->symlink_name);
-	info->symlink_name = brasero_utils_check_for_parent_symlink (target);
-	g_free (target);
-
-	/* we check for circular dependency here :
-	 * if the target is one of the parent of symlink */
-	size = strlen (info->symlink_name);
-	if (!strncmp (info->symlink_name, uri, size)
-	&& (*(uri + size) == '/' || *(uri + size) == '\0'))
-		return FALSE;
-	
-	return TRUE;
-}
-
 /* Copied from glib-2.8.3 (glib.c) but slightly
  * modified to use only the first 64 bytes */
 gboolean
@@ -657,23 +512,19 @@ void
 brasero_utils_launch_app (GtkWidget *widget,
 			  GSList *list)
 {
-	char *uri;
-	char *mime;
+	gchar *uri;
+	gchar *mime;
 	GList *uris;
 	GSList *item;
-	char *escaped_uri;
 	GnomeVFSResult result;
 	GnomeVFSMimeApplication *application;
 
 	for (item = list; item; item = item->next) {
 		uri = item->data;
-		escaped_uri = gnome_vfs_escape_host_and_path_string (uri);
 
-		mime = gnome_vfs_get_mime_type (escaped_uri);
-		if (!mime) {
-			g_free (escaped_uri);
+		mime = gnome_vfs_get_mime_type (uri);
+		if (!mime)
 			continue;
-		}
 
 		application = gnome_vfs_mime_get_default_application (mime);
 		g_free (mime);
@@ -697,11 +548,10 @@ brasero_utils_launch_app (GtkWidget *widget,
 
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
-			g_free (escaped_uri);
 			continue;
 		}
 
-		uris = g_list_prepend (NULL, escaped_uri);
+		uris = g_list_prepend (NULL, uri);
 		result = gnome_vfs_mime_application_launch (application, uris);
 		g_list_free (uris);
 
@@ -720,15 +570,14 @@ brasero_utils_launch_app (GtkWidget *widget,
 			gtk_window_set_title (GTK_WINDOW (dialog), _("File error"));
 
 			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-							 _("application %s can't be started."),
-							 application->name);
+								 _("application %s can't be started."),
+								 application->name);
 
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
 		}
 
 		gnome_vfs_mime_application_free (application);
-		g_free (escaped_uri);
 	}
 }
 void
@@ -923,6 +772,10 @@ brasero_utils_get_use_info_notebook (void)
 	GtkWidget *notebook;
 	GtkWidget *event_box;
 	GtkWidget *first_use;
+	GtkWidget *alignment;
+	gchar     *message_add, *message_add_header;
+	gchar     *message_remove, *message_remove_header;
+	gchar	  *first_use_message;
 
 	notebook = gtk_notebook_new ();
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
@@ -968,61 +821,78 @@ brasero_utils_get_use_info_notebook (void)
 
 	gtk_container_add (GTK_CONTAINER (frame), event_box);
 
-	first_use = gtk_label_new (_("<span foreground='grey50'><big>To add files to this project you can:\n</big>"
-				     "copy them (from nautilus for example) and paste them here\n"
-				     "select them and click on the add button\n"
-				     "double click on them\n"
-				     "drag them here\n"
-				     "\n\n\n\n"
-				     "<big>To remove files from this project you can:\n</big>"
-				     "select them, click right and choose \"Remove\" from the menu\n"
-				     "select them and click on the remove button\n"
-				     "select them and press \"Delete\" key\n"
-//				     "drag them out"
-				     "</span>"));
+	alignment = gtk_alignment_new (0.50, 0.30, 0, 0);
+	gtk_container_add (GTK_CONTAINER (event_box), alignment);
+
+	/* Translators: this messages will appear as a list of possible
+	 * actions, like:
+	 *   To add/remove files you can:
+         *      * perform action one
+         *      * perform action two
+	 * The full message will be showed in the main area of an empty
+	 * project, suggesting users how to add and remove items to project.
+	 * You simply have to translate messages in the best form
+         * for a list of actions. */
+	message_add_header = g_strconcat ("<big>", _("To add files to this project you can:"), "\n</big>", NULL);
+	message_add = g_strconcat ("\t* ", _("click the \"Add\" button to show the selection pane"), "\n",
+				   "\t* ", _("select files in selection pane and click the \"Add\" button"), "\n",
+				   "\t* ", _("drag files in this area from the selection pane or from the file manager"), "\n",
+				   "\t* ", _("double click on files in the selection pane"), "\n",
+
+				   "\t* ", _("copy files (from file manager for example) and paste in this area"), "\n",
+				   NULL);
+
+	message_remove_header = g_strconcat ("<big>", _("To remove files from this project you can:"), "\n</big>", NULL);
+	message_remove = g_strconcat ("\t* ", _("click on the \"Remove\" button to remove selected items in this area"), "\n",
+				      "\t* ", _("drag and release items out from this area"), "\n",
+				      "\t* ", _("select items in this area, and choose \"Remove\" from context menu"), "\n",
+				      "\t* ", _("select items in this area, and press \"Delete\" key"), "\n",
+				      NULL);
+	
+
+	first_use_message = g_strconcat ("<span foreground='grey50'>",
+					 message_add_header, message_add,
+					 "\n\n\n",
+					 message_remove_header, message_remove,
+					 "</span>", NULL);
+	first_use = gtk_label_new (first_use_message);
+	g_free (first_use_message);
 
 	gtk_misc_set_padding (GTK_MISC (first_use), 24, 0);
-	gtk_label_set_justify (GTK_LABEL (first_use), GTK_JUSTIFY_CENTER);
+	gtk_label_set_justify (GTK_LABEL (first_use), GTK_JUSTIFY_LEFT);
 	gtk_label_set_use_markup (GTK_LABEL (first_use), TRUE);
-	gtk_container_add (GTK_CONTAINER (event_box), first_use);
+	gtk_container_add (GTK_CONTAINER (alignment), first_use);
 	gtk_event_box_set_above_child (GTK_EVENT_BOX (event_box), TRUE);
 
+	g_free (message_add_header);
+	g_free (message_add);
+	g_free (message_remove_header);
+	g_free (message_remove);
+
 	return notebook;
 }
 
-gchar *
-brasero_utils_validate_uri (const gchar *uri, gboolean escaped)
+gchar*
+brasero_utils_validate_utf8 (const gchar *name)
 {
-	gchar *retval;
-	gchar *scheme;
+	gchar *retval, *ptr;
+	const gchar *invalid;
 
-	scheme = gnome_vfs_get_uri_scheme (uri);
-	if (!scheme) {
-		gchar *full_path;
+	if (!name)
+		return NULL;
 
-		/* this looks like a local path, turn it into a URI */
-		if (uri [0] != G_DIR_SEPARATOR) {
-			/* we must have a full local path for the next function */
-			full_path = g_build_path (G_DIR_SEPARATOR_S,
-						  g_get_current_dir (),
-						  uri,
-						  NULL);
-		}
-		else
-			full_path = g_strdup (uri);
+	if (g_utf8_validate (name, -1, &invalid))
+		return NULL;
 
-		/* this is a local filename so we can't expect it to be escaped.
-		 * This function also takes care to escaped the URI. */
-		retval = gnome_vfs_get_uri_from_local_path (full_path);
-		g_free (full_path);
-	}
-	else if (escaped) {
-		g_free (scheme);
-		retval = g_strdup (uri);
-	}
-	else {
-		g_free (scheme);
-		retval = gnome_vfs_escape_host_and_path_string (uri);
+	retval = g_strdup (name);
+	ptr = retval + (invalid - name);
+	*ptr = '_';
+	ptr++;
+
+	while (!g_utf8_validate (ptr, -1, &invalid)) {
+		ptr = (gchar*) invalid;
+		*ptr = '?';
+		ptr ++;
 	}
 
 	return retval;
